##############################################################################
## File         : canvas.tcl
## Title        : Drawing features
## Date         : 15/05/1997
## Version      : 1.10
## Last Changes : 03/04/1998
##
###############################################################################
set drawret -1
set cvFill gray95 
set cv(0,Top)  0

proc fgl_CanvasSet     { n i j V} { }
proc fgl_CanvasState   { n E } { }
proc fgl_CanvasSelect  { pid n } { }
proc fgl_CanvasFocus   { n I } { }

option add *FglCanvas.relief          sunken            userDefault
option add *FglCanvas.borderWidth          1            userDefault
option add *fglCanvasFont           $fglFont            userDefault

set fglCanvasfont [option get . fglCanvasFont "" ]

proc fgl_CanvasCreate {  n y x sx sy } {
  global topWnd dy dx
  set n "[string range $n 0 [expr [string length $n ] - [string length "$y"] -2 ]]"
  set w $topWnd.c${n}
  canvas $w \
    -class FglCanvas\
    -height [expr $sy * $dy] \
    -width  [expr $sx * $dx] 
  place $w \
    -x [fglFormX $x] \
    -y [fglFormY $y]
}

proc fgl_CanvasSelect { pid n } {
  global topWnd cv 
  set cv($pid,Top) $topWnd.c${n}
  set cv($pid,Fill) gray95
  set cv($pid,Fx) [expr [lindex [ $cv($pid,Top) configure -width ] 4 ] / 1000.0 ]
  set cv($pid,Fy) [expr [lindex [ $cv($pid,Top) configure -height] 4 ] / 1000.0 ]
#  set cv(pid) $pid
}

proc fgl_CvClear { pid } {
  global cv
  $cv($pid,Top) delete all
}

proc cvX { pid x } {
  global cv
  return [expr $x * $cv($pid,Fx)] 
}
proc cvY { pid y } {
  global cv
  return [expr (1000.0 - $y) * $cv($pid,Fy)] }

proc fgl_CvText { pid x y t a } {
  global cv drawret fglCanvasfont
  set drawret [$cv($pid,Top) create text [cvX $pid $x] [cvY $pid $y]\
    -text $t -anchor $a \
    -font $fglCanvasfont \
    ]
}

proc fgl_CvLine { pid x y dx dy w l } {
  global cv drawret 
  if { $l == 0 } {
    set drawret [$cv($pid,Top) create line [cvX $pid $x] [cvY $pid $y] \
      [cvX $pid [expr $x + $dx]] [cvY $pid [expr $y + $dy]] \
      -width $w -fill $cv($pid,Fill)]
  } else {
    set drawret [$cv($pid,Top) create line [cvX $pid $x] [cvY $pid $y] \
      [cvX $pid [expr $x + $dx]] [cvY $pid [expr $y + $dy]] \
      -width $w ]
  }
}

proc fgl_CvSetFillColor { pid c } {
  global cv
  set cv($pid,Fill) $c
}
proc fgl_CvRectangle { pid x y dx dy } {
  global cv drawret
  set drawret  [$cv($pid,Top) create rectangle \
    [cvX $pid $x] [cvY $pid $y] \
    [cvX $pid [expr $x + $dx]] [cvY $pid [expr $y + $dy]] \
    -fill $cv($pid,Fill)]
}
############################################################################
proc fgl_CvOval { pid x y rx ry } {
  global cv drawret
  set drawret [$cv($pid,Top) create oval \
    [cvX $pid $x] [cvY $pid $y] \
    [cvX $pid [expr $x + $rx]] [cvY $pid [expr $y + $ry]] \
    -fill $cv($pid,Fill)]
}
proc fgl_CvCircle { pid x y r } {
  global cv drawret
  set drawret [$cv($pid,Top) create oval \
    [cvX $pid $x] [cvY $pid $y] \
    [cvX $pid [expr $x+ $r]] [expr [cvY $pid $y]+[cvX $pid $r]] \
    -fill $cv($pid,Fill)]
}
proc fgl_CvPolygon { pid l } {
  global cv drawret
  for {set i 0} {$i< [expr [llength $l] - 1 ]} { incr i 2} {
    regexp {([^\.]+).*} [cvY $pid [lindex $l $i]] a b
    regexp {([^\.]+).*} [cvX $pid [lindex $l [expr $i + 1]]] a d
    lappend t $d
    lappend t $b
  }
  set a "$cv($pid,Top) create polygon $t -fill $cv($pid,Fill)"
  set drawret [eval $a]
}
proc fgl_CvArc { pid x y d s l } {
  global cv drawret
  set drawret [$cv($pid,Top) create arc \
     [cvX $pid $x] [cvY $pid $y] \
     [cvX $pid [expr $x + $d]] [expr [cvY $pid $y] + [cvX $pid $d]] \
     -start $s -extent $l \
     -fill $cv($pid,Fill)]
}
proc fgl_CvBind { pid n b k } {
   global cv
   if { [string compare $k Control-c] == 0 } {
     $cv($pid,Top) bind $n $b "fglX6 $cv($pid,Top) $k"
   } elseif { [string compare $k "nokey"] == 0 } {
     $cv($pid,Top) bind $n $b ""
   } else {
     $cv($pid,Top) bind $n $b "fglXX $cv($pid,Top) $k"
   }
}
